@@ -187,6 +187,7 @@ class KNNAttention(nn.Module):
         num_retrieved_memories = 32,
         xl_max_memories = 0.,
         attn_scale_init = 20,
+        gate_output = False
     ):
         super().__init__()
         self.heads = heads
@@ -204,6 +205,8 @@ class KNNAttention(nn.Module):
         self.to_kv = nn.Linear(dim, dim_head * 2, bias = False)
         self.to_out = nn.Linear(inner_dim, dim, bias = False)
 
+        self.output_gate = nn.Parameter(torch.zeros(1)) if gate_output else None
+
     def forward(
         self,
         x,
@@ -282,8 +285,15 @@ class KNNAttention(nn.Module):
         # combine heads and project out
 
         out = rearrange(out, 'b h n d -> b n (h d)')
+        out = self.to_out(out)
 
-        return self.to_out(out), new_xl_kv_memories
+        # use flamingo styled gating of output, so that memorizing transformers can be gated into an existing LLM
+        # preparation to add this to block-recurrent-transformer-pytorch, for the pinnacle of long context attention network
+
+        if exists(self.output_gate):
+            out = out * self.output_gate.tanh()
+
+        return out, new_xl_kv_memories
 
 # main class
 
